﻿Imports System.Runtime.CompilerServices
Imports Infragistics.Win.UltraWinGrid

Module Extensions

    'IMPORTANT NOTE: When the grid is disabled, the KeepFocus does not work !!!

    Private _codefield As String = ""
    Private _selected_code As Long = -1
    Private _selectionchanging As Boolean = False

    <Extension()>
    Public Sub KeepFocusSelectedWhenGroupedBy(ByVal grd As Infragistics.Win.UltraWinGrid.UltraGrid, ByVal codefield As String)
        _codefield = codefield.Trim
        If _codefield.Length > 0 Then
            AddHandler grd.AfterSelectChange, AddressOf grd_AfterSelectChange
            AddHandler grd.AfterRowActivate, AddressOf grd_AfterRowActivate
        Else
            RemoveHandler grd.AfterSelectChange, AddressOf grd_AfterSelectChange
            RemoveHandler grd.AfterRowActivate, AddressOf grd_AfterRowActivate
        End If
    End Sub

    Private Sub grd_AfterSelectChange(sender As Object, e As AfterSelectChangeEventArgs)
        If TypeOf (sender) Is Infragistics.Win.UltraWinGrid.UltraGrid Then
            _selectionchanging = True
            DoSelectRow(DirectCast(sender, Infragistics.Win.UltraWinGrid.UltraGrid).Rows)
            DirectCast(sender, Infragistics.Win.UltraWinGrid.UltraGrid).Selected.Rows.Clear()
        End If
    End Sub

    Private Sub grd_AfterRowActivate(sender As Object, e As EventArgs)
        If _selectionchanging Then Debug.WriteLine("After RowActivate Abort") : Exit Sub
        If TypeOf (sender) Is Infragistics.Win.UltraWinGrid.UltraGrid Then
            Dim grd = DirectCast(sender, Infragistics.Win.UltraWinGrid.UltraGrid)
            If grd.ActiveRow.IsDataRow Then _selected_code = CLng(CType(CType(grd.ActiveRow.ListObject, DataRowView).Row, DataRow).Item(_codefield))
        End If
    End Sub

    Private Function DoSelectRow(input As Object) As Boolean
        If TypeOf (input) Is Infragistics.Win.UltraWinGrid.SelectedRowsCollection Then
            For Each row In DirectCast(input, SelectedRowsCollection)
                DoSelectRow(row)
            Next
        ElseIf TypeOf (input) Is Infragistics.Win.UltraWinGrid.UltraGridGroupByRow Then
            Dim grprow = DirectCast(input, UltraGridGroupByRow)
            For Each row In grprow.Rows
                DoSelectRow(row)
            Next
        ElseIf TypeOf (input) Is Infragistics.Win.UltraWinGrid.RowsCollection Then
            For Each row In DirectCast(input, RowsCollection)
                If (DoSelectRow(row)) Then
                    If row.HasParent AndAlso row.ParentRow.IsExpandable Then row.ParentRow.ExpandAll()
                End If
            Next
        ElseIf TypeOf (input) Is UltraGridRow Then
            Dim urow = DirectCast(input, UltraGridRow)
            If urow.IsDataRow Then
                Dim dr = DirectCast(urow.ListObject, DataRowView).Row
                If CLng(dr(_codefield)) = _selected_code Then
                    urow.Activate()
                    _selectionchanging = False
                    Return True
                End If
            End If
        End If
        Return False
    End Function


    <Extension()>
    Public Function GetActiveRow_KeyValue(ByVal grd As Infragistics.Win.UltraWinGrid.UltraGrid, key_field As String) As Long
        If Not IsNothing(grd.ActiveRow) AndAlso
           grd.ActiveRow.Cells.Exists(key_field) AndAlso
           Not IsNothing(grd.ActiveRow.Cells(key_field).Value) Then
            If TypeOf (grd.ActiveRow.Cells(key_field).Value) Is Integer Then
                Return CInt(grd.ActiveRow.Cells(key_field).Value)
            ElseIf TypeOf (grd.ActiveRow.Cells(key_field).Value) Is Long Then
                Return CLng(grd.ActiveRow.Cells(key_field).Value)
            End If
        End If
        Return -1
    End Function

    <Extension()>
    Public Sub SelectActiveRow(ByVal grd As Infragistics.Win.UltraWinGrid.UltraGrid, key_value As Long, key_field As String)
        If Not IsNothing(grd.Rows) Then
            For Each row In grd.Rows
                If DoSelectRow(row) Then Exit For
            Next
        End If
    End Sub


    Public Class srtComparerTypeDoc
        Implements IComparer

        Public Function Compare(ByVal x As Object, ByVal y As Object) As Integer Implements System.Collections.IComparer.Compare

            Dim xCell As UltraGridCell = DirectCast(x, UltraGridCell)
            xCell = xCell.Row.Cells("typedoc")
            Dim yCell As UltraGridCell = DirectCast(y, UltraGridCell)
            yCell = yCell.Row.Cells("typedoc")

            Dim text1 As String = xCell.Value.ToString()
            Dim text2 As String = yCell.Value.ToString()

            Return String.Compare(text1, text2, True)
        End Function

    End Class

   

End Module
